//#################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de
// 14.01.2006
//
//
// This version uses TWO buffers. One small for text data and
// one big for graphics data.
//
//#################################################################

#include <windows.h>
#include <stdio.h>
#include "stdafx.h"

#include "serial.h"

//###############################################################
// Write Buffers for better USB performance !

unsigned char textbuffer[TEXT_BUFFER_SIZE];
unsigned int textbuffercount=0;
unsigned int textbuffersize=1;  //change this value in config.ini !

unsigned char gfxbuffer[GFX_BUFFER_SIZE];
unsigned int gfxbuffercount=0;
unsigned int gfxbuffersize=64;  //change this value in config.ini !

//###############################################################




	// variables used with the com port
	BOOL			bPortReady;
	DCB				dcb;
	COMMTIMEOUTS	CommTimeouts;
    char errmsg[256];

//####################################################################################
// Send and clean the serial graphics buffer
int SerialFlushGfxBuffer(HANDLE *hCom)
//####################################################################################
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;

    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return 1;

    if(gfxbuffercount>0 && gfxbuffercount<=GFX_BUFFER_SIZE)
	{
	 bWriteRC = WriteFile(*hCom, gfxbuffer, gfxbuffercount, &iBytesWritten,NULL);
	 gfxbuffercount=0;
	}

 return 0;
}

//####################################################################################
// Fill the graphics send buffer and send it if full.
// This gives more performance with USB-Serial adapters !
int SerialPutToGfxBuffer(HANDLE *hCom, unsigned char dat)
//####################################################################################
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;

    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return 1;

    if(gfxbuffercount<GFX_BUFFER_SIZE) gfxbuffer[gfxbuffercount]=dat;
    gfxbuffercount++;

	if(gfxbuffercount>=gfxbuffersize) //send the buffer if full
	{
	 bWriteRC = WriteFile(*hCom, gfxbuffer, gfxbuffersize, &iBytesWritten,NULL);
	 gfxbuffercount=0;
    }

	return 0;
}

//####################################################################################
// Send and clean the serial text buffer
int SerialFlushTextBuffer(HANDLE *hCom)
//####################################################################################
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;

    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return 1;

    if(textbuffercount>0 && textbuffercount<=TEXT_BUFFER_SIZE)
	{
	 bWriteRC = WriteFile(*hCom, textbuffer, textbuffercount, &iBytesWritten,NULL);
	 textbuffercount=0;
	}

 return 0;
}

//####################################################################################
// Fill the text send buffer and send it if full.
// This gives more performance with USB-Serial adapters !
int SerialPutToTextBuffer(HANDLE *hCom, unsigned char dat)
//####################################################################################
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;

    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return 1;

    if(textbuffercount<TEXT_BUFFER_SIZE) textbuffer[textbuffercount]=dat;
    textbuffercount++;

	if(textbuffercount>=textbuffersize) //send the buffer if full
	{
	 bWriteRC = WriteFile(*hCom, textbuffer, textbuffersize, &iBytesWritten,NULL);
	 textbuffercount=0;
    }

	return 0;
}

//####################################################################################
HANDLE SerialInit(char *ComPortName, int BaudRate) 
//####################################################################################
{
    DWORD error;
	HANDLE hCom;

    textbuffercount=0;
    gfxbuffercount=0;

	hCom = CreateFile(ComPortName, 
		GENERIC_READ | GENERIC_WRITE,
		0, // exclusive access
		NULL, // no security
		OPEN_EXISTING,
		0, // no overlapped I/O
		NULL); // null template 

    if(hCom==INVALID_HANDLE_VALUE)
	{
        error=GetLastError();
		sprintf(errmsg,"##################### CreateFile() for %s , Baudrate %d failed ! Error %ld #####################",ComPortName,BaudRate,error);
        MessageBox(NULL,errmsg,NULL,MB_OK);
		return 0;
	}

	bPortReady = SetupComm(hCom, 4096, 4096); // set buffer sizes
    if(bPortReady==0)
	{
        error=GetLastError();
		sprintf(errmsg,"##################### SetupComm() for %s failed ! Error %ld #####################",ComPortName,error);
        MessageBox(NULL,errmsg,NULL,MB_OK);
		CloseHandle(hCom); 
		return 0;
	}

	bPortReady = GetCommState(hCom, &dcb);
    if(bPortReady==0)
	{
        error=GetLastError();
		sprintf(errmsg,"##################### GetCommState() for %s failed ! Error %ld #####################",ComPortName,error);
        MessageBox(NULL,errmsg,NULL,MB_OK);
		CloseHandle(hCom); 
		return 0;
	}

	dcb.BaudRate = BaudRate;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
//	dcb.Parity = EVENPARITY;
	dcb.StopBits = ONESTOPBIT;
	dcb.fAbortOnError = TRUE;

	// set XON/XOFF
	dcb.fOutX = FALSE;					// XON/XOFF off for transmit
	dcb.fInX	= FALSE;					// XON/XOFF off for receive
	// set RTSCTS
	dcb.fOutxCtsFlow = FALSE;					// turn on CTS flow control
	dcb.fRtsControl = RTS_CONTROL_DISABLE;	// 
//	dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;	// 
	// set DSRDTR
	dcb.fOutxDsrFlow = FALSE;					// turn on DSR flow control
//	dcb.fDtrControl = DTR_CONTROL_ENABLE;	// 
	dcb.fDtrControl = DTR_CONTROL_DISABLE;	// 
//	dcb.fDtrControl = DTR_CONTROL_HANDSHAKE;	// 

	bPortReady = SetCommState(hCom, &dcb);
    if(!bPortReady) return NULL;

	// Communication timeouts are optional

	bPortReady = GetCommTimeouts (hCom, &CommTimeouts);
    if(!bPortReady) return NULL;

	CommTimeouts.ReadIntervalTimeout = 1000; //5000;
	CommTimeouts.ReadTotalTimeoutConstant = 1000; //5000;
	CommTimeouts.ReadTotalTimeoutMultiplier = 1000;
	CommTimeouts.WriteTotalTimeoutConstant = 1000; //5000;
	CommTimeouts.WriteTotalTimeoutMultiplier = 1000;

	bPortReady = SetCommTimeouts (hCom, &CommTimeouts);
    if(!bPortReady) return 0;

	PurgeComm(hCom,PURGE_RXABORT | PURGE_RXCLEAR);
    PurgeComm(hCom,PURGE_TXABORT | PURGE_TXCLEAR);

//This is for my COM port powered LCD's
//Set DTR and RTS high to power the LCD
	EscapeCommFunction(hCom, SETDTR);
	EscapeCommFunction(hCom, SETRTS);

	return hCom;
}

//####################################################################################
void SerialPutc(HANDLE *hCom, unsigned char txchar)
//####################################################################################
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;
	
    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return;

	bWriteRC = WriteFile(*hCom, &txchar, 1, &iBytesWritten,NULL);
}

/*
//####################################################################################
char SerialGetc(HANDLE *hCom)
//####################################################################################
{
	char rxchar;
	BOOL	bReadRC;
	static	DWORD	iBytesRead;

    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return 0;

	bReadRC = ReadFile(*hCom, &rxchar, 1, &iBytesRead, NULL);

	return rxchar;
}

//####################################################################################
char* SerialGets(HANDLE *hCom)
//####################################################################################
{
	static char rxstring[256];
	char errstring[256];
	char c;
	int pos = 0;
	DWORD err;

    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return NULL;

	while(pos <= 255) {
		c = SerialGetc(hCom);
		err=GetLastError();
		if(err!=ERROR_SUCCESS)
		{
		 switch(err)
		 {
		  case ERROR_COUNTER_TIMEOUT: sprintf(errstring,"Timeout der seriellen Schnittstelle !\n"); break;
		  default: sprintf(errstring,"Error Code %d",err); break;
		 }
		 
		 MessageBox(NULL,errstring,NULL,MB_OK);
		 break;
		}
//		if(c == '\r') continue;		// discard carriage return
//		if(c == '\n') break;
//		if(c == 0x0D) break;		// discard carriage return
//		if(c == 0x0A) continue;
		if(c == 0x0A) break;		// discard carriage return
		if(c == 0x0D) continue;
		if(c>=0x20) rxstring[pos++] = c;
	}
	rxstring[pos] = 0;

	return	rxstring;
}

//####################################################################################
void SerialPuts(HANDLE *hCom, unsigned char *txstring)
//####################################################################################
{
	BOOL	bWriteRC;
	static	DWORD	iBytesWritten;

    if(*hCom==0 || *hCom==INVALID_HANDLE_VALUE) return;

	bWriteRC = WriteFile(*hCom, txstring, strlen((char *) txstring), &iBytesWritten,NULL);

}

//####################################################################################
void sleep( int _wait)
//####################################################################################
{
	clock_t goal;
	goal = clock() + _wait;
	while( goal > clock() );
}

*/